
set frame HTTPD
set frametitle "HTTPD Server"
set icontitle "HTTPD"

set  httpdstarted 0

proc make-frame {} {
   global env

set rows 6
set cols 90

  :frame [self frame-name] :label "[self frame-title]"  :icon-name "[self icon-title]"

  :panel P1 :layout V :padx 20
    :layout H; :col-gap 20
     :button LOAD :label "Load source file" :command click_load
     :button CLEAR :command ":modify LIST :empty T"

  :panel P2 :layout V
    :galley LIST :rows $rows :cols $cols :deaf T  :font variable

   open-frame

  :galley_size LIST $rows $cols
}

proc click-configure {W} {:galley_resize LIST}

proc click_load {} {midas-browser R do_load}

proc do_load {args} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file

   set f [file join $directory $file]
   InsertLog "loading $f"
   set z [catch {source $f} m]
   if {$z != 0} then {InsertLog "$m"}
}

proc InsertLog {s} {
    global httpdstarted
    if {$httpdstarted == 1} {:modify LIST :strings $s\n} else {puts $s}
}

set Retry 0
proc click-Retry {v} {global Retry; set Retry $v}

proc check_for_experiment {} {
    global env Retry

    set z [catch {set env(EXPERIMENTBASE)}]
    if {$z == 1 || ![file isdirectory $env(EXPERIMENTBASE)]} then {

        :frame [self frame-name] :label "[self frame-title]" \
           :icon-image "@$env(ICONHOME)/default" \
           :icon-name "[self icon-title]" :resize F

        :panel P :layout T :padx 10 :pady 10
        :layout H; :col-gap 10; :row-gap 10

        :message M  :bold T :justify center :label "\
You are not connected to a useable experiment\n\
It is essential that you are connected to an experiment before starting the Httpd server\n\
Please correct this and then click on Continue."

        :next-row T

        :button CONTINUE :label Continue :w 12 :command "click-Retry 0"
        :button ABORT    :label Abort    :w 12 :command "click-Retry 1"

        :show T

        tkwait variable Retry

         destroy-frame

        if {$Retry == 0} then {return 1} else {return -1}
    } else {return 0}
}

proc start args {
   global httpdstarted env Options
   upvar #0 browserdirectory directory

   for {set i 0} {$i < 100} {incr i} {
       set rc [check_for_experiment]
       if {$rc < 0}  return
       if {$rc == 0} break
   }

   if {$httpdstarted == 0} then {
      set httpdstarted 1  
      
      set directory [file join $env(MIDASBASE) TclHttpd tcl]

      load_Options

      if {![info exists Options(ServerPort)]}    {set Options(ServerPort) "8115"}
      if {![info exists Options(ServerCode)]}    {set Options(ServerCode) "RunControl"}
      if {![info exists Options(MakeFrame)]}     {set Options(MakeFrame) "0"}
      if {![info exists Options(MessageLoggerServerHost)]}     {set Options(MessageLoggerServerHost) "none"}
      if {![info exists Options(MessageLoggerServerPort)]}     {set Options(MessageLoggerServerPort) "$Options(ServerPort)"}

      update_Options

      if {$Options(MakeFrame) != 0} {make-frame}

       global MessageLoggerServerHost MessageLoggerServerPort
       set MessageLoggerServerHost $Options(MessageLoggerServerHost)
       set MessageLoggerServerPort $Options(MessageLoggerServerPort)

namespace inscope :: {

puts "Lauching Httpd for Prospectus control"

      set argv "-debug 1 -port $Options(ServerPort) -docRoot [file join $env(MIDASBASE) Prospectus Html]"
      cd [file join $env(MIDASBASE) TclHttpd bin]
      source httpd_Server.tcl

      Setting BGColor "BlanchedAlmond"
      Setting TextColor "red"

      Setting ProjectName  ProSPECTus
      Setting ProjectImage prospectus60.jpg
      Setting ProjectURL   "http://npg.dl.ac.uk/NPG/Projects/Prospectus.html"

  Setting Option3Name "Project Home Page"
  Setting Option3URL  "http://npg.dl.ac.uk/NPG/Projects/Prospectus.html"

  Setting Option4Name "Server Software Home Page"
  Setting Option4URL  "http://npg.dl.ac.uk/MIDAS/download/WebServices.html"


    if {$Options(MessageLoggerServerHost) == "localhost"} {
       set MIDAS_MessageLoggerServer_UseFile 0
       set MIDAS_MessageLoggerServer_UseELog 0
       if {[info exists Options(ElogCmd)]} {set MIDAS_MessageLoggerServer_ElogCmd $Options(ElogCmd)}
       package require MessageLoggerServer
    }
    
      package require MessageLoggerClient
      package require $Options(ServerCode)
      global MIDAS_DataAcquisitionControlServer_Verbose
      if {[info exists MIDAS_DataAcquisitionControlServer_Verbose]} then {
          set MIDAS_DataAcquisitionControlServer_Verbose 0
      }

      unset started
}

   } else {
      if {$Options(MakeFrame) != 0} {open-frame}
   }
}



#    The Options are experiment specific and normally used by SYS
#    However the same code as for the session preferences is used which allows for wider use in the future.

proc get_Option_file {} {
    global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

proc put_Option_file {lines} {
   global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf w]} m]
    if {$z != 0} then {
        insert-log "Unable to access $pf - $m"
        return 1
    }

    foreach item $lines {
      if {$item != ""} {puts $f $item}
    }

    close $f
    return 0
}

#    save all Options for the current task
proc update_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
   }

   lappend lines "## $key"
   foreach item [array names Options] {
       lappend lines "$item $Options($item)"
   }
   lappend lines "##"

   set z [put_Option_file $lines]
   return $z
}

#    load all Options for the current task
proc load_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           set Options([head $item]) [tail $item]
       }
   }
}

#    return task Options
proc get_Options {task} {
   global env

   set lines [get_Option_file]
   set key $task
   set range [find-keyword $lines $key]

   set Options {}

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           lappend Options [list [first $item] [second $item]]
       }
   } else {
       lappend Options [list Options.filename  "[file join $env(HOME) MidasOptions Options]"]
   }
   return $Options
}


