#    most of this task is now taken from the standard application

set VERSION "CAEN"

#    put some default environment variables before loading common 


if {[info exists env(VMEServer)]} then {set resource $env(VMEServer)} else {set resource VME}
set VMEserver $resource

if {[info exists env(RDOUTserver)]} then {set resource $env(RDOUTserver)} else {set resource VME}
set RDOUTserver $resource

set LinuxDirectory  [file join $env(MIDASBASE) Data_Acq bin Linux64]
set DAProgram    "CAENExecV1 XXX"
set DADirectory  $LinuxDirectory


loadsource  sys.common

rename click-setup __setup
proc click-setup {arg} {
    if {$arg == 0} {__setup 1} else {__setup $arg}
}

set frametitle "MIDAS Data Acquisition Experiment Control"

set VMEPresent 0
set VMEEnable 0

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_Data"]

lappend registerlist [list NetVar.V1724_TLow     generic  "-n V1724_TLow -t int"]
lappend registerlist [list NetVar.V1724_THigh    generic  "-n V1724_THigh -t int"]
lappend registerlist [list NetVar.V1724_TraceOpt generic  "-n V1724_TraceOpt -t int"]

loadsource sys.DA     ;#  for common data acquisition support

loadsource sys.WWW    ;#   for control via Web services

loadsource sys.2D    ;# for 2D histogrammming feature 

set Supported_Modules [list V1724 V785 V775 V1190 V1290 V767 VME]


proc Dump_Raw_Data {} {
    global Options

    set Options(Include.S9418) 0  ;# so common code OK for non CAEN modules
    set Options(Include.VF48) 0
    set Options(Include.MDI2) 0

    Dump_Raw_VME_Data
}

proc Dump_Raw_VME_Data {} {
    global Options
    global VMEserver

   set info ""

   set CAEN_ModuleNum 0
   foreach m [list V1724 V785] {

#       set maxmodule [inform vmecontrol inquire [string tolower $m]count]
#       if {[string length $maxmodule] > 2} {set maxmodule 0}

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/[string tolower $m]#$i.Status'  ?object)" } r]
      if {$z!=0} then {break}
      incr maxmodule
   }

       incr CAEN_ModuleNum $maxmodule
   }

   for {set CAEN_ModuleIndex 1} {$CAEN_ModuleIndex <= $CAEN_ModuleNum} {incr CAEN_ModuleIndex} {

      execute-command 19 "Getting Raw Data for CAEN module $CAEN_ModuleIndex" "1 $CAEN_ModuleIndex" 2
      set data [read-register NetVar.EXEC.Response]

      if {[string range [first $data] 0 1] == "0x"} then {set CAEN_ModuleType [expr [first $data]]} else {set CAEN_ModuleType [expr 0x[first $data]]}

      append info "CAEN module $CAEN_ModuleIndex is type V$CAEN_ModuleType\n"
      catch {append info "[Dump_Raw_VME_Data_V$CAEN_ModuleType [lreplace $data 0 0]]\n"}

   }

   if {[info exists Options(Include.VME)] && $Options(Include.VME) == 1} then {append info [Dump_Raw_VME_Data_VME]}

#   midas-information "$info"
    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $info :font fixed :rows 70 :cols 85
    :show T
}

proc redisplay-txenable {} {
   global  RDOUTEnable txEnable  TSPresent
   if {$TSPresent == 0} then {return}
   if {$RDOUTEnable == 0} then {:modify TXENABLE :value 0 :colours [list red red]; return}
   set txEnable [read-register NetVar.DACQ.Ctxenable]
   :modify TXENABLE :value $txEnable
#   click-special 20
#   set txState [read-register NetVar.EXEC.Parameter]
#   if {$txState == 0} then {
#       :modify TXENABLE :colours [list red red]
#   } else {
#       :modify TXENABLE :colours [list red green]
#   }
    :modify TXENABLE :colours [list grey grey]
}

proc Custom_SetupA  {opt} {

   define_VME_addresses   ;#  setup VME 
}

proc Custom_PreGo {} {
    global VMEserver Options

    for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v1724#$i.Status'  ?object)" } m]
      if {$z!=0} then {break}

      write-registers v1724#$i.Control 0     ;# set BERR enable
      write-registers v1724#$i.BLTEventNum 1
      write-registers v1724#$i.SoftwareSyncReset 0  
    }
  
    write-register NetVar.V1724_TLow $Options(V1724_TLow)
    write-register NetVar.V1724_THigh $Options(V1724_THigh)
    write-register NetVar.V1724_TraceOpt $Options(V1724_TraceOpt)
}

set Retry 0
proc click-Retry {v} {global Retry; set Retry $v}

proc check_for_VME {msg} {
    global env Retry

        :frame [self frame-name].vme :label "[self frame-title]" \
           :icon-image "@$env(ICONHOME)/default" \
           :icon-name "[self icon-title]" :resize F

        :panel P :layout T :padx 10 :pady 10
        :layout H; :col-gap 10; :row-gap 10

        :message M  :bold T :justify center :label "$msg"

        :next-row T

        :button CONTINUE :label Continue :w 12 :command "click-Retry 0"

        :show T

        tkwait variable Retry

         destroy-frame [self frame-name].vme
}

proc Custom_PreCheck {opt} {
    global VMEserver

     set z -1
     while {$z != 0} {
         set z [catch {eg "(register :name '/$VMEserver/HasBeenConfigured'  ?value)" } m]
         if {$z!=0 || $m != 1} then {
             check_for_VME "\
The VME server has not been configured\n\
It may recently have been restarted\n\
Start a Web Browser and access http://localhost:8015/VME/Config/VMEConfig.tml\n\
Click on \"Reload\" \"Restore Configuration\" and \"Activate Configuration\"\n\
Then CONTINUE from here\n\
"
             set z -1
         }
     }

     set z -1
     while {$z != 0} {
         set z [catch {eg "(register :name '/$VMEserver/HasBeenRestored'  ?value)" } m]
         if {$z!=0 || $m != 1} then {
             check_for_VME "\
The VME module setup has not been restored\n\
The VME server may recently have been restarted\n\
Start a Web Browser and access http://localhost:8015/VME/V1724/V1724.tml\n\
Click on \"Reload\" \"Restore Settings for all modules\"\n\
Then CONTINUE from here\n\
"
             set z -1
         }
     }

}


#    support for Web Service access  -  provides application specific support

proc do_Setup {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Setup:  Acquisition state = $MIDAS_DataAcquisitionCode"

    after 100 WWW_Setup

   set MIDAS_DataAcquisitionCode 3
}

proc WWW_Setup {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-setup 1} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc do_Reset {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Reset:  Acquisition state = $MIDAS_DataAcquisitionCode"

    after 100 WWW_Reset

   set MIDAS_DataAcquisitionCode 7
}

proc WWW_Reset {} {
     global MIDAS_DataAcquisitionCode

    if {[da_state] != 0} {set z [catch {click-da 0} m]}

        set MIDAS_DataAcquisitionCode 2
}

proc do_Stop {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Stop:  Acquisition state = $MIDAS_DataAcquisitionCode"

    after 100 WWW_Stop

   set MIDAS_DataAcquisitionCode 5
}

proc WWW_Stop {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-da 0} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc do_Go {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Go:  Acquisition state = $MIDAS_DataAcquisitionCode"

    after 100 WWW_Go

   set MIDAS_DataAcquisitionCode 4
}

proc WWW_Go {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-da 1} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc Get_State {args} {
     global MIDAS_DataAcquisitionCode

#   puts "Get_State:  Acquisition state = $MIDAS_DataAcquisitionCode"

    if {$MIDAS_DataAcquisitionCode == 8} {set MIDAS_DataAcquisitionCode [da_state]}

     return $MIDAS_DataAcquisitionCode
}

proc do_SetOption {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_SetOption:  Acquisition state = $MIDAS_DataAcquisitionCode"

    switch [first $args] {

        default     {puts "Unknown Acquisition Option [first $args]"}
    }   
}

proc do_GetOption {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_GetOption:  Acquisition state = $MIDAS_DataAcquisitionCode"

    switch [first $args] {

        default     {puts "Unknown Acquisition Option [first $args]"}
    }   

    return "none"
}

